/*
 * This file is part of the Distribution library.
 *
 * Copyright (c) 2015-2016 LIN3S <info@lin3s.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @author Beñat Espiña <bespina@lin3s.com>
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _path = require('path');

var _precss = require('precss');

var _precss2 = _interopRequireDefault(_precss);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _Modernizr = require('./Plugins/Modernizr');

var _Modernizr2 = _interopRequireDefault(_Modernizr);

var _SvgStore = require('./Plugins/SvgStore');

var _SvgStore2 = _interopRequireDefault(_SvgStore);

var _UglifyJs = require('./Plugins/UglifyJs');

var _UglifyJs2 = _interopRequireDefault(_UglifyJs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isProdEnvironment = function isProdEnvironment(options) {
  return typeof options !== 'undefined' && options.env === 'prod';
};

var optionsOf = function optionsOf(plugin, options) {
  var defaultOptions = void 0;
  if (typeof options.plugins !== 'undefined' && typeof options.plugins[plugin] !== 'undefined') {
    defaultOptions = options.plugins[plugin];
  }

  return defaultOptions;
};

var getPlugins = function getPlugins(options) {
  var plugins = [(0, _SvgStore2.default)(optionsOf('svgStore', options)), new _extractTextWebpackPlugin2.default(options.output.cssRoute), new _webpack2.default.LoaderOptionsPlugin({
    options: {
      postcss: [(0, _autoprefixer2.default)(options.postcss.autoprefixer), _precss2.default],
      sassLoader: {
        includePaths: [(0, _path.join)(__dirname, options.input.scss)]
      }
    }
  })];

  if (isProdEnvironment(options)) {
    plugins.push((0, _UglifyJs2.default)(optionsOf('uglifyjs', options)));
    plugins.push((0, _Modernizr2.default)(optionsOf('modernizr', options)));
  }

  return plugins;
};

var getLoaders = function getLoaders(include, options) {
  var loaders = [{
    test: /\.js$/,
    loader: 'babel',
    include: include,
    query: {
      compact: false
    }
  }, {
    test: /\.json$/,
    loader: 'json'
  }, {
    test: /\.(s?css)$/,
    loader: _extractTextWebpackPlugin2.default.extract({
      fallbackLoader: 'style-loader',
      loader: ['css-loader', 'postcss-loader', 'sass-loader']
    })
  }];

  if (typeof options.module !== 'undefined' && typeof options.module.loaders !== 'undefined') {
    loaders.push(options.module.loaders);
  }

  return loaders;
};

exports.default = function (customOptions) {
  var include = (0, _path.join)(__dirname, customOptions.input.base);

  return function (webpackOptions) {
    var options = typeof webpackOptions === 'undefined' ? customOptions : Object.assign(webpackOptions, customOptions);

    return {
      entry: options.entry,
      output: {
        path: options.output.jsPath,
        filename: options.output.jsFilename
      },
      module: {
        loaders: getLoaders(include, options)
      },
      resolve: {
        modules: [include, 'node_modules'],
        extensions: ['.js', '.json'],
        alias: options.alias
      },
      plugins: getPlugins(options),
      devtool: isProdEnvironment(options) ? false : 'source-map'
    };
  };
};